OccupiedInterval

Struct OccupiedInterval 

struct OccupiedInterval {
    range: Range<i16>,
    state: CellOccupancyState,
}
A run of occupied cells within a single track. The range is in OriginZero line coordinates along the track (so for a row track, the range spans column lines and vice versa).

Fields

range: Range<i16>

The range of cells covered by this interval (start line..end line in OriginZero coordinates)

state: CellOccupancyState

The occupancy state of every cell within this interval

Implementations

Source

impl OccupiedInterval

fn overlaps(&self, range: &Range<i16>) -> bool

Whether this interval overlaps the given range